@@ -5,8 +5,8 @@
 #
 # Copyright (c) 2007-2008 Peng Huang <shawn.p.huang@gmail.com>
 # Copyright (c) 2009 Hideaki ABE <abe.sendai@gmail.com>
-# Copyright (c) 2010-2013 Takao Fujiwara <takao.fujiwara1@gmail.com>
-# Copyright (c) 2007-2013 Red Hat, Inc.
+# Copyright (c) 2010-2017 Takao Fujiwara <takao.fujiwara1@gmail.com>
+# Copyright (c) 2007-2017 Red Hat, Inc.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -27,6 +27,12 @@ __all__ = (
         'ThumbShiftSegment',
     )
 
+import sys
+
+from gi import require_version as gi_require_version
+gi_require_version('GLib', '2.0')
+gi_require_version('IBus', '1.0')
+
 from gi.repository import GLib
 from gi.repository import IBus
 
@@ -35,182 +41,182 @@ import segment
 _THUMB_BASIC_METHOD = 'base'
 
 _table_static = {
-    'q': [u'。', u'',   u'ぁ'],
-    'w': [u'か', u'が', u'え'],
-    'e': [u'た', u'だ', u'り'],
-    'r': [u'こ', u'ご', u'ゃ'],
-    't': [u'さ', u'ざ', u'れ'],
+    'q': ['。', '',   'ぁ'],
+    'w': ['か', 'が', 'え'],
+    'e': ['た', 'だ', 'り'],
+    'r': ['こ', 'ご', 'ゃ'],
+    't': ['さ', 'ざ', 'れ'],
 
-    'y': [u'ら', u'よ', u'ぱ'],
-    'u': [u'ち', u'に', u'ぢ'],
-    'i': [u'く', u'る', u'ぐ'],
-    'o': [u'つ', u'ま', u'づ'],
-    'p': [u'，',  u'ぇ', u'ぴ'],
-    '@': [u'、', u'',   u''],
-    '[': [u'゛', u'゜', u''],
+    'y': ['ら', 'よ', 'ぱ'],
+    'u': ['ち', 'に', 'ぢ'],
+    'i': ['く', 'る', 'ぐ'],
+    'o': ['つ', 'ま', 'づ'],
+    'p': ['，',  'ぇ', 'ぴ'],
+    '@': ['、', '',   ''],
+    '[': ['゛', '゜', ''],
 
-    'a': [u'う', u'',   u'を'],
-    's': [u'し', u'じ', u'あ'],
-    'd': [u'て', u'で', u'な'],
-    'f': [u'け', u'げ', u'ゅ'],
-    'g': [u'せ', u'ぜ', u'も'],
+    'a': ['う', '',   'を'],
+    's': ['し', 'じ', 'あ'],
+    'd': ['て', 'で', 'な'],
+    'f': ['け', 'げ', 'ゅ'],
+    'g': ['せ', 'ぜ', 'も'],
 
-    'h': [u'は', u'み', u'ば'],
-    'j': [u'と', u'お', u'ど'],
-    'k': [u'き', u'の', u'ぎ'],
-    'l': [u'い', u'ょ', u'ぽ'],
-    ';': [u'ん', u'っ', u''],
+    'h': ['は', 'み', 'ば'],
+    'j': ['と', 'お', 'ど'],
+    'k': ['き', 'の', 'ぎ'],
+    'l': ['い', 'ょ', 'ぽ'],
+    ';': ['ん', 'っ', ''],
 
-    'z': [u'．',  u'',   u'ぅ'],
-    'x': [u'ひ', u'び', u'ー'],
-    'c': [u'す', u'ず', u'ろ'],
-    'v': [u'ふ', u'ぶ', u'や'],
-    'b': [u'へ', u'べ', u'ぃ'],
+    'z': ['．',  '',   'ぅ'],
+    'x': ['ひ', 'び', 'ー'],
+    'c': ['す', 'ず', 'ろ'],
+    'v': ['ふ', 'ぶ', 'や'],
+    'b': ['へ', 'べ', 'ぃ'],
 
-    'n': [u'め', u'ぬ', u'ぷ'],
-    'm': [u'そ', u'ゆ', u'ぞ'],
-    ',': [u'ね', u'む', u'ぺ'],
-    '.': [u'ほ', u'わ', u'ぼ'],
-    '/': [u'・', u'ぉ', u''],
+    'n': ['め', 'ぬ', 'ぷ'],
+    'm': ['そ', 'ゆ', 'ぞ'],
+    ',': ['ね', 'む', 'ぺ'],
+    '.': ['ほ', 'わ', 'ぼ'],
+    '/': ['・', 'ぉ', ''],
 
-    '1': [u'1',  u'',   u'？'],
-    '2': [u'2',  u'',   u'／'],
-    '4': [u'4',  u'',   u'「'],
-    '5': [u'5',  u'',   u'」'],
+    '1': ['1',  '',   '？'],
+    '2': ['2',  '',   '／'],
+    '4': ['4',  '',   '「'],
+    '5': ['5',  '',   '」'],
 
-    '6': [u'6',  u'［',  u''],
-    '7': [u'7',  u'］',  u''],
-    '8': [u'8',  u'（',  u''],
-    '9': [u'9',  u'）',  u''],
-    '\\': [u'￥', u'',  u''],
+    '6': ['6',  '［',  ''],
+    '7': ['7',  '］',  ''],
+    '8': ['8',  '（',  ''],
+    '9': ['9',  '）',  ''],
+    '\\': ['￥', '',  ''],
 }
 
 _nicola_j_table_static = {
-    ':': [u'：', u'',   u''],
-    '@': [u'、', u'',   u''],
-    '[': [u'゛', u'゜', u''],
-    ']': [u'」', u'',   u''],
-    '8': [u'8',  u'（', u''],
-    '9': [u'9',  u'）', u''],
-    '0': [u'0',  u'',   u''],
+    ':': ['：', '',   ''],
+    '@': ['、', '',   ''],
+    '[': ['゛', '゜', ''],
+    ']': ['」', '',   ''],
+    '8': ['8',  '（', ''],
+    '9': ['9',  '）', ''],
+    '0': ['0',  '',   ''],
 }
 
 _nicola_a_table_static = {
-    ':': [u'：', u'',   u''],
-    '@': [u'＠', u'',   u''],
-    '[': [u'、', u'',   u''],
-    ']': [u'゛', u'゜', u''],
-    '8': [u'8',  u'',   u''],
-    '9': [u'9',  u'（', u''],
-    '0': [u'0',  u'）', u''],
+    ':': ['：', '',   ''],
+    '@': ['＠', '',   ''],
+    '[': ['、', '',   ''],
+    ']': ['゛', '゜', ''],
+    '8': ['8',  '',   ''],
+    '9': ['9',  '（', ''],
+    '0': ['0',  '）', ''],
 }
 
 _nicola_f_table_static = {
-    ':': [u'、', u'',   u''],
-    '@': [u'＠', u'',   u''],
-    '[': [u'゛', u'゜', u''],
-    ']': [u'」', u'',   u''],
-    '8': [u'8',  u'（', u''],
-    '9': [u'9',  u'）', u''],
-    '0': [u'0',  u'',   u''],
+    ':': ['、', '',   ''],
+    '@': ['＠', '',   ''],
+    '[': ['゛', '゜', ''],
+    ']': ['」', '',   ''],
+    '8': ['8',  '（', ''],
+    '9': ['9',  '）', ''],
+    '0': ['0',  '',   ''],
 }
 
 _kb231_j_fmv_table_static = {
-    '3': [u'3',  u'',   u'～'],
-    '0': [u'0',  u'『', u''],
-    '-': [u'-',  u'』', u''],
-    '=': [u'=',  u'',   u''],
+    '3': ['3',  '',   '～'],
+    '0': ['0',  '『', ''],
+    '-': ['-',  '』', ''],
+    '=': ['=',  '',   ''],
 }
 
 _kb231_a_fmv_table_static = {
-    '3': [u'3',  u'',   u'～'],
-    '0': [u'0',  u'）', u''],
-    '-': [u'-',  u'『', u''],
-    '=': [u'=',  u'』', u''],
+    '3': ['3',  '',   '～'],
+    '0': ['0',  '）', ''],
+    '-': ['-',  '『', ''],
+    '=': ['=',  '』', ''],
 }
 
 _kb231_f_fmv_table_static = {
-    '3': [u'3',  u'',   u'～'],
-    '0': [u'0',  u'『', u''],
-    '-': [u'-',  u'』', u''],
-    '=': [u'=',  u'',   u''],
+    '3': ['3',  '',   '～'],
+    '0': ['0',  '『', ''],
+    '-': ['-',  '』', ''],
+    '=': ['=',  '',   ''],
 }
 
 _kb611_j_fmv_table_static = {
-    '`':  [u'‘', u'',   u''],
-    '^':  [u'々', u'£',  u''],
-    ':':  [u'：', u'',   u''],
-    '@':  [u'、', u'¢',  u''],
-    '[':  [u'゛', u'゜', u''],
+    '`':  ['‘', '',   ''],
+    '^':  ['々', '£',  ''],
+    ':':  ['：', '',   ''],
+    '@':  ['、', '¢',  ''],
+    '[':  ['゛', '゜', ''],
     # keysyms are same and keycodes depend on the platforms.
     #'￥': [u'￥', u'¬',  u''],
-    '\\': [u'￥', u'¦',  u''],
+    '\\': ['￥', '¦',  ''],
 }
 
 _kb611_a_fmv_table_static = {
-    '`':  [u'々', u'',   u'£'],
-    ':':  [u'：', u'',   u''],
-    '@':  [u'＠', u'',   u''],
-    '[':  [u'、', u'¢',  u''],
+    '`':  ['々', '',   '£'],
+    ':':  ['：', '',   ''],
+    '@':  ['＠', '',   ''],
+    '[':  ['、', '¢',  ''],
     #'￥': [u'￥', u'¬',  u''],
-    '\\': [u'￥', u'¦',  u''],
+    '\\': ['￥', '¦',  ''],
 }
 
 _kb611_f_fmv_table_static = {
-    '`':  [u'‘', u'',   u''],
-    '^':  [u'々', u'£',  u''],
-    ':':  [u'、', u'¢',  u''],
-    '@':  [u'＠', u'',   u''],
-    '[':  [u'゛', u'゜', u''],
+    '`':  ['‘', '',   ''],
+    '^':  ['々', '£',  ''],
+    ':':  ['、', '¢',  ''],
+    '@':  ['＠', '',   ''],
+    '[':  ['゛', '゜', ''],
     #'￥': [u'￥', u'¬',  u''],
-    '\\': [u'￥', u'¦',  u''],
+    '\\': ['￥', '¦',  ''],
 }
 
 _shift_table = {
-    'H': u'ぱ',
-    'X': u'ぴ',
-    'V': u'ぷ',
-    'B': u'ぺ',
-    '>': u'ぽ',
+    'H': 'ぱ',
+    'X': 'ぴ',
+    'V': 'ぷ',
+    'B': 'ぺ',
+    '>': 'ぽ',
 }
 
 table_static = {}
 r_table_static = {}
 
-for k in _table_static.keys():
+for k in list(_table_static.keys()):
     table_static[ord(k)] = _table_static[k]
     for c in _table_static[k]:
         r_table_static[c] = k
 
 kana_voiced_consonant_rule = {
-    u'か゛' : u'が',
-    u'き゛' : u'ぎ',
-    u'く゛' : u'ぐ',
-    u'け゛' : u'げ',
-    u'こ゛' : u'ご',
-    u'さ゛' : u'ざ',
-    u'し゛' : u'じ',
-    u'す゛' : u'ず',
-    u'せ゛' : u'ぜ',
-    u'そ゛' : u'ぞ',
-    u'た゛' : u'だ',
-    u'ち゛' : u'ぢ',
-    u'つ゛' : u'づ',
-    u'て゛' : u'で',
-    u'と゛' : u'ど',
-    u'は゛' : u'ば',
-    u'ひ゛' : u'び',
-    u'ふ゛' : u'ぶ',
-    u'へ゛' : u'べ',
-    u'ほ゛' : u'ぼ',
-    u'は゜' : u'ぱ',
-    u'ひ゜' : u'ぴ',
-    u'ふ゜' : u'ぷ',
-    u'へ゜' : u'ぺ',
-    u'ほ゜' : u'ぽ',
+    'か゛' : 'が',
+    'き゛' : 'ぎ',
+    'く゛' : 'ぐ',
+    'け゛' : 'げ',
+    'こ゛' : 'ご',
+    'さ゛' : 'ざ',
+    'し゛' : 'じ',
+    'す゛' : 'ず',
+    'せ゛' : 'ぜ',
+    'そ゛' : 'ぞ',
+    'た゛' : 'だ',
+    'ち゛' : 'ぢ',
+    'つ゛' : 'づ',
+    'て゛' : 'で',
+    'と゛' : 'ど',
+    'は゛' : 'ば',
+    'ひ゛' : 'び',
+    'ふ゛' : 'ぶ',
+    'へ゛' : 'べ',
+    'ほ゛' : 'ぼ',
+    'は゜' : 'ぱ',
+    'ひ゜' : 'ぴ',
+    'ふ゜' : 'ぷ',
+    'へ゜' : 'ぺ',
+    'ほ゜' : 'ぽ',
 }
 
-_UNFINISHED_HIRAGANA = set(u'かきくけこさしすせそたちつてとはひふへほ')
+_UNFINISHED_HIRAGANA = set('かきくけこさしすせそたちつてとはひふへほ')
 
 class ThumbShiftKeyboard:
     def __init__(self, prefs=None):
@@ -225,8 +231,7 @@ class ThumbShiftKeyboard:
         self.__layout = 0
         self.__fmv_extension = 2
         self.__handakuten = False
-        self.__thumb_typing_rule_section_base = None
-        self.__thumb_typing_rule_section = None
+        self.__thumb_typing_rule_method = None
         self.__init_thumb_typing_rule()
         self.__init_layout_table()
         if self.__prefs != None:
@@ -236,40 +241,33 @@ class ThumbShiftKeyboard:
     def __init_thumb_typing_rule(self):
         prefs = self.__prefs
         if prefs == None:
-            self.__thumb_typing_rule_section = None
+            self.__thumb_typing_rule_method = None
             return
-        method = prefs.get_value('thumb_typing_rule', 'method')
+        method = prefs.get_value('thumb-typing-rule', 'method')
         if method == None:
             method = _THUMB_BASIC_METHOD
-        self.__thumb_typing_rule_section_base = 'thumb_typing_rule'
-        self.__thumb_typing_rule_section = \
-            self.__thumb_typing_rule_section_base + '/' + method
-        if self.__thumb_typing_rule_section not in prefs.sections():
-            self.__thumb_typing_rule_section = None
+        self.__thumb_typing_rule_method = method
+        keymap = prefs.get_value('thumb-typing-rule', 'list')
+        if self.__thumb_typing_rule_method not in keymap.keys():
+            self.__thumb_typing_rule_method = None
 
     def __init_layout_table(self):
         if self.__table != {}:
             self.__table.clear()
         if self.__r_table != {}:
             self.__r_table.clear()
-        section_base = self.__thumb_typing_rule_section_base
-        section = self.__thumb_typing_rule_section
-        if section != None:
+        method = self.__thumb_typing_rule_method
+        if method != None:
             prefs = self.__prefs
-            for k in prefs.keys(section):
-                value = prefs.get_value(section, k)
-                ch = prefs.typing_from_config_key(k)
-                if ch == '':
-                    continue
-                self.__set_bus_table(ch, value)
-            for k in prefs.get_value(section_base, 'newkeys'):
-                value = prefs.get_value_direct(section, k)
+            keymap = prefs.get_value('thumb-typing-rule', 'list')[method]
+            for k in keymap.keys():
+                value = keymap.get(k)
                 ch = prefs.typing_from_config_key(k)
                 if ch == '':
                     continue
                 self.__set_bus_table(ch, value)
         else:
-            for k in _table.keys():
+            for k in list(_table.keys()):
                 self.__table[ord(k)] = _table_static[k]
                 for c in _table_static[k]:
                     self.__r_table[c] = k
@@ -278,11 +276,8 @@ class ThumbShiftKeyboard:
         prefs = self.__prefs
         if value == None or len(value) != 3:
             return
-        value = [prefs.unicode(prefs.str(value[0])),
-                 prefs.unicode(prefs.str(value[1])),
-                 prefs.unicode(prefs.str(value[2]))]
-        if value[0] == u'' and \
-                value[1] == u'' and value[2] == u'':
+        if value[0] == '' and \
+                value[1] == '' and value[2] == '':
             return
         self.__table[ord(key)] = value
         for c in value:
@@ -307,26 +302,19 @@ class ThumbShiftKeyboard:
             sub_table = f_table
         if method == None or sub_table == None:
             return
-        section_base = self.__thumb_typing_rule_section_base
-        section = self.__thumb_typing_rule_section
-        sub_section = section_base + '/' + method
-        if section != None:
+        method = self.__thumb_typing_rule_method
+        if method != None:
             prefs = self.__prefs
-            for k in prefs.keys(sub_section):
-                value = prefs.get_value(sub_section, k)
-                ch = prefs.typing_from_config_key(k)
-                if ch == '':
-                    continue
-                self.__set_bus_table(ch, value)
-            for k in prefs.get_value(section_base, method + '_newkeys'):
-                value = prefs.get_value_direct(sub_section, k)
+            keymap = prefs.get_value('thumb-typing-rule', 'list')[method]
+            for k in keymap.keys():
+                value = keymap.get(k)
                 ch = prefs.typing_from_config_key(k)
                 if ch == '':
                     continue
                 self.__set_bus_table(ch, value)
         else:
-            for k in sub_table.keys():
-                self.__table[ord(unicode(k))] = sub_table[k]
+            for k in list(sub_table.keys()):
+                self.__table[ord(str(k))] = sub_table[k]
                 for c in sub_table[k]:
                     self.__r_table[c] = k
 
@@ -360,11 +348,11 @@ class ThumbShiftKeyboard:
     def __reset_shift_table(self, init):
         self.__reset_extension_table(init)
         if self.__handakuten:
-            for k in _shift_table.keys():
+            for k in list(_shift_table.keys()):
                 self.__shift_table[ord(k)] = _shift_table[k]
                 self.__r_table[_shift_table[k]] = k
         elif self.__shift_table != {}:
-            for k in _shift_table.keys():
+            for k in list(_shift_table.keys()):
                 if ord(k) in self.__shift_table:
                     del self.__shift_table[ord(k)]
                 if _shift_table[k] in self.__r_table:
@@ -411,7 +399,6 @@ class ThumbShiftKeyboard:
                                 0, 1024, 0)[3]
             layout_list = prop.split('\0')
         except TypeError:
-            import sys
             print >> sys.stderr, \
               'This problem is fixed in the latest gobject-introspection'
             print >> sys.stderr, \
@@ -437,9 +424,9 @@ class ThumbShiftKeyboard:
                                 GLib.SpawnFlags.SEARCH_PATH_FROM_ENVP,
                                 None, None)
         if not ret:
-            print >> sys.stderr, std_error
+            print(std_error.decode('utf-8'), file=sys.stderr)
             return layout
-        for line in std_out.split('\n'):
+        for line in std_out.decode('utf-8').split('\n'):
             if line.startswith('layout:'):
                 data = line.split()[1]
                 if data == 'jp':
@@ -452,18 +439,18 @@ class ThumbShiftKeyboard:
                     layout = 2
                 elif data.find('japan:') >= 0:
                     layout = 0
-        return 0
+        return layout
 
     def __reset_layout_and_handakuten(self):
-        mode = self.__prefs.get_value('thumb', 'keyboard_layout_mode')
+        mode = self.__prefs.get_value('thumb', 'keyboard-layout-mode')
         layout = 0
         if mode == 1:
             layout = self.__get_xkb_layout()
         else:
-            layout = self.__prefs.get_value('thumb', 'keyboard_layout')
+            layout = self.__prefs.get_value('thumb', 'keyboard-layout')
         self.set_layout(layout)
 
-        fmv_extension = self.__prefs.get_value('thumb', 'fmv_extension')
+        fmv_extension = self.__prefs.get_value('thumb', 'fmv-extension')
         self.set_fmv_extension(fmv_extension)
         handakuten = self.__prefs.get_value('thumb', 'handakuten')
         self.set_handakuten(handakuten)
@@ -544,19 +531,19 @@ class ThumbShiftKeyboard:
         return self.__table.get(key, fallback)
 
     def get_chars(self):
-        return self.__table.keys()
+        return list(self.__table.keys())
 
     def get_r_char(self, key, fallback=None):
         return self.__r_table.get(key, fallback)
 
     def get_r_chars(self):
-        return self.__r_table.keys()
+        return list(self.__r_table.keys())
 
     def get_shift_char(self, key, fallback=None):
         return self.__shift_table.get(key, fallback)
 
     def get_shift_chars(self):
-        return self.__shift_table.keys()
+        return list(self.__shift_table.keys())
 
 
 class ThumbShiftSegment(segment.Segment):
@@ -565,13 +552,13 @@ class ThumbShiftSegment(segment.Segment):
     _thumb_typing_rule_section = None
     _r_table = {}
 
-    def __init__(self, enchars=u'', jachars=u''):
+    def __init__(self, enchars='', jachars=''):
         if not jachars:
-            if u'!' <= enchars <= u'~':
+            if '!' <= enchars <= '~':
                 jachars = segment.unichar_half_to_full(enchars)
             else:
                 jachars = enchars
-                enchars = self._r_table.get(jachars, u'')
+                enchars = self._r_table.get(jachars, '')
         super(ThumbShiftSegment, self).__init__(enchars, jachars)
 
     @classmethod
@@ -580,38 +567,31 @@ class ThumbShiftSegment(segment.Segment):
         if prefs == None:
             cls._thumb_typing_rule_section = None
             return
-        method = prefs.get_value('thumb_typing_rule', 'method')
+        method = prefs.get_value('thumb-typing-rule', 'method')
         if method == None:
             method = _THUMB_BASIC_METHOD
-        cls._thumb_typing_rule_section_base = 'thumb_typing_rule'
-        cls._thumb_typing_rule_section = \
-            cls._thumb_typing_rule_section_base + '/' + method
-        if cls._thumb_typing_rule_section not in prefs.sections():
-            cls._thumb_typing_rule_section = None
+        cls._thumb_typing_rule_method = method
+        keymap = prefs.get_value('thumb-typing-rule', 'list')
+        if cls._thumb_typing_rule_method not in keymap.keys():
+            cls._thumb_typing_rule_method = None
         cls._init_layout_table()
 
     @classmethod
     def _init_layout_table(cls):
         if cls._r_table != {}:
             cls._r_table.clear()
-        section_base = cls._thumb_typing_rule_section_base
-        section = cls._thumb_typing_rule_section
-        if section != None:
+        method = cls._thumb_typing_rule_method
+        if method != None:
             prefs = cls._prefs
-            for k in prefs.keys(section):
-                value = prefs.get_value(section, k)
-                ch = prefs.typing_from_config_key(k)
-                if ch == '':
-                    continue
-                cls._set_bus_table(ch, value)
-            for k in prefs.get_value(section_base, 'newkeys'):
-                value = prefs.get_value_direct(section, k)
+            keymap = prefs.get_value('thumb-typing-rule', 'list')[method]
+            for k in keymap.keys():
+                value = keymap.get(k)
                 ch = prefs.typing_from_config_key(k)
                 if ch == '':
                     continue
                 cls._set_bus_table(ch, value)
         else:
-            for k in _table.keys():
+            for k in list(_table.keys()):
                 for c in _table_static[k]:
                     cls._r_table[c] = k
 
@@ -620,11 +600,8 @@ class ThumbShiftSegment(segment.Segment):
         prefs = cls._prefs
         if value == None or len(value) != 3:
             return
-        value = [prefs.unicode(prefs.str(value[0])),
-                 prefs.unicode(prefs.str(value[1])),
-                 prefs.unicode(prefs.str(value[2]))]
-        if value[0] == u'' and \
-                value[1] == u'' and value[2] == u'':
+        if value[0] == '' and \
+                value[1] == '' and value[2] == '':
             return
         for c in value:
             cls._r_table[c] = key
@@ -633,31 +610,31 @@ class ThumbShiftSegment(segment.Segment):
         return not (self._jachars in _UNFINISHED_HIRAGANA)
 
     def append(self, enchar):
-        if enchar == u'\0' or enchar == u'':
+        if enchar == '\0' or enchar == '':
             return []
         text = self._jachars + enchar
         jachars = kana_voiced_consonant_rule.get(text, None)
         if jachars:
-            self._enchars = self._enchars + self._r_table.get(enchar, u'')
+            self._enchars = self._enchars + self._r_table.get(enchar, '')
             self._jachars = jachars
             return []
         return [ThumbShiftSegment(enchar)]
 
     def prepend(self, enchar):
-        if enchar == u'\0' or enchar == u'':
+        if enchar == '\0' or enchar == '':
             return []
-        if self._jachars == u'':
+        if self._jachars == '':
             if 0x21 <= enchars <= 0x7e:
                 self._enchars = enchar
                 self._jachars = segment.unichar_half_to_full(enchars)
             else:
-                self._enchars = self._r_table.get(enchar, u'')
+                self._enchars = self._r_table.get(enchar, '')
                 self._jachars = enchar
             return []
         return [ThumbShiftSegment(enchar)]
 
     def pop(self, index=-1):
-        self._enchars = u''
-        self._jachars = u''
+        self._enchars = ''
+        self._jachars = ''
         return
 
